/* **************************************************************************
 * @version $Id: BMFtest.c,v 0.2 2006/04/03 10:33:00 legolas558 Exp $
 *
 * File:		BMFtest.c
 * Content:		main program source for testing my OpenGL BMfont
 *				implementation
 * Notes:		uses GLFW framework compiled into a static library
 *
 * Copyright(c) 2006 by legolas558
 *
 * https://sourceforge.net/projects/oglbmfont
 *
 * This software is released under a BSD license. See LICENSE.TXT
 * You must accept the license before using this software.
 *
 * **************************************************************************/

#include "BMFtest.h"

#define	SCREEN_W	800			//screen width
#define	SCREEN_H	600			//screen height

void	ogl_setup_2d() {
	// set 2d mode
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glOrtho(0, SCREEN_W, 0, SCREEN_H, 0, 1);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	// invert y axis since OpenGL uses it updown
	glScalef(1, -1, 1);
	glTranslatef(0, -SCREEN_H, 0);
}

int WINAPI WinMain(HINSTANCE h1, HINSTANCE h2, LPSTR s, int n) {
	ogl_font	myfont;								// the font object we will use
	GLuint		screen_rect[4];						// an array containing the screen
													// bounding rectangle

	zmem_init();									// dummy heap debugger, ignore it

	if (!glfwInit())								// initializes GLFW framework
		return -1;
	atexit(glfwTerminate);							// setup the cleaner hook

    if( !glfwOpenWindow(SCREEN_W, SCREEN_H, 5,6,6,8, 8, 0, GLFW_WINDOW ) )
		return -2;									// open a rendering context

    glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );	// clear screen & depth buffer
    glAlphaFunc( GL_GREATER, 0 );							// set the alpha transparency
    glEnable( GL_ALPHA_TEST );								// and turn it on

	if (!ogl_font_load(&myfont, "final.fnt"))			// loads a BMF font
		return -3;

	screen_rect[_X] = screen_rect[_Y] = 0;					// sets up a screen rectangle
	screen_rect[_W] = SCREEN_W; screen_rect[_H] = SCREEN_H;

	ogl_setup_2d();											// (see above) sets up the (x,y)

	ogl_printf(&myfont, 0, 40, "Nel mezzo del cammin di nostra vita (%dx%d)", SCREEN_W, SCREEN_H);
	ogl_printf_al(&myfont, al_center, (GLuint *)&screen_rect, "mi ritrovai per una selva oscura");
	myfont.scale = 2.f;
	ogl_printf(&myfont, 0, 0, "che la diritta via era smarrita");
	glfwSwapBuffers();										// flushes primitives to screen
	
	glfwSleep(1.7f);											// wait some time

	ogl_font_destroy(&myfont);								// free the object

	return 0;
}
